//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


////////////////////////////////////////////////////////////////////
// Nes Mapper #9:  MMC2
// This mapper is used only on the U.S. versions of Punch-Out
////////////////////////////////////////////////////////////////////

#ifndef NES_MAPPER_9
#define NES_MAPPER_9

#include "../NesBaseMapper.h"
#include "../NesPPUMemory.h"


// Derive a special PPU memory class that maintains the swappable latch value
class MMC2PPUMemory : public NesPPUMemory
{
public:
	MMC2PPUMemory::MMC2PPUMemory() {		
		mintLatchSelect1 = 0xFD;
		mintLatchSelect2 = 0xFE;
		mintLatchSelect3 = 0xFE;
		mintLatchSelect4 = 0xFE;
		mintLatchValue1 = mintLatchValue2 = mintLatchValue3 = mintLatchValue4 = 0;
	}

	void checkLatch(int addr) {
		if (addr >= 0x1000) {
			if ( (addr & 0xFF0) == 0xFD0) {				
				if (mintLatchSelect2 != 0xFD) {
					mMapper->swapCHRBank4(0x1000, mintLatchValue3);					
				}
				mintLatchSelect2 = 0xFD;
			} else if ( (addr & 0xFF0) == 0xFE0) {				
				if (mintLatchSelect2 != 0xFE) {
					mMapper->swapCHRBank4(0x1000, mintLatchValue4);			
				}
				mintLatchSelect2 = 0xFE;
			}
		} else {
			if ( (addr & 0xFF0) == 0xFD0) {				
				if (mintLatchSelect1 != 0xFD) mMapper->swapCHRBank4(0x0000, mintLatchValue1);
				mintLatchSelect1 = 0xFD;				
			} else if ( (addr & 0xFF0) == 0xFE0) {				
				if (mintLatchSelect1 != 0xFE) mMapper->swapCHRBank4(0x0000, mintLatchValue2);				
				mintLatchSelect1 = 0xFE;			
			}
		}		
	}

	virtual Uint8 read(Uint16 &addr, bool blnCPURead) {
		// Check for latch swap, then call the super method to do standard processing
		checkLatch(addr);
		return NesPPUMemory::read(addr, blnCPURead);
	}
	
};


class NesMapper9 : public NesBaseMapper
{
public:	
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper9(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU);
	~NesMapper9();	
	void write(Uint16 addr, unsigned char data);
	void initMemory();
	
	
	///////////////////////////////
	// Class Data Members
	///////////////////////////////
	// This data member stores the 8k bank of PRG-ROM that is currently
	// loaded into memory at $8000, and is used to prevent swappnig data in that
	// is already there
	int intBankMap8, intChrLow, intChrHigh;
	int intLatchReg1, intLatchReg2;
};

#endif 
