//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


////////////////////////////////////////////////////////////////////
// Nes Mapper #21:  Konami VRC4
// This mapper is used on several Japanese titles by Konami, such as
// Wai Wai World 2 and Gradius 2
////////////////////////////////////////////////////////////////////

#ifndef NES_MAPPER_21
#define NES_MAPPER_21

#include "../NesBaseMapper.h"
#include "../NesPPU.h"



class NesMapper21 : public NesBaseMapper
{
public:
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper21() { };
	NesMapper21(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU);
	~NesMapper21() { };
	virtual void initMemory();
	virtual void write(Uint16 addr, unsigned char data);
	
	bool blnIrqEnable;
	int intIrqLatch, intIrqCount, intEnableLatch;

protected:
	unsigned char chrbanks[8];
	int intSwitchSpace;
};



// We need a special PPU class for scanline counting/IRQ generation
class VRC4PPU : public NesPPU {
	virtual void calcScanLine() {
		// Draw scanline as usual
		NesPPU::calcScanLine();

		// Get a handle to our mapper object, this is pretty convoluted
		NesMapper21 *mmc = (NesMapper21 *) VRAM->mMapper;
	
		// Do IRQ logic
		if (mmc->blnIrqEnable) {
			mmc->intIrqCount++;
			if (mmc->intIrqCount == 0xFF) {
				mmc->mPpu->timers.blnOtherIRQ = true;
				mmc->mPpu->timers.intOtherPix = 0*3;
				mmc->intIrqCount = mmc->intIrqLatch;
			}				
		}
	}
};

#endif 
