//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper232.cpp - Implementation of mapper #232 (Quattro Games)
//////////////////////////////////////////////////////////////////////

#include "NesMapper232.h"


NesMapper232::NesMapper232(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
}


void NesMapper232::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);				// First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1);	// Last 16k bank to $C000
	}
		
	reg[0] = 0x0C;
	reg[1] = 0x00;	
}


void NesMapper232::write(Uint16 addr, unsigned char data) {		
	// Select PRGROM
	if (addr == 0x9000) { 
		reg[0] = (data & 0x18) >> 1;
	} else if (addr >= 0xA000) {
		reg[1] = data & 0x03;
	}
	swapPRGBank8(0x8000, ((reg[0] | reg[1])*2 + 0) % (bytPRGCount << 1) );
	swapPRGBank8(0xA000, ((reg[0] | reg[1])*2 + 1) % (bytPRGCount << 1) );
	swapPRGBank8(0xC000, ((reg[0] | 0x03)*2 + 0) % (bytPRGCount << 1) );
	swapPRGBank8(0xE000, ((reg[0] | 0x03)*2 + 1) % (bytPRGCount << 1) );

	//SetPROM_8K_Bank( 6, (reg[0]|0x03)*2+0 );
	//SetPROM_8K_Bank( 7, (reg[0]|0x03)*2+1 );
}



//-------------------------------------------------------------------
// Procedure: saveState
//-------------------------------------------------------------------
// Description: Saves the state of the mapper to the file stream
//-------------------------------------------------------------------
void NesMapper232::saveState(FILE *stream) {
	NesBaseMapper::saveState(stream);
	fwrite(&reg[0], sizeof(reg[0]), 1, stream);		// misc
	fwrite(&reg[1], sizeof(reg[1]), 1, stream);		// misc
}





//-------------------------------------------------------------------
// Procedure: loadState
//-------------------------------------------------------------------
// Description: Loads the state of the mapper from the file stream
//-------------------------------------------------------------------
void NesMapper232::loadState(FILE *stream) {
	NesBaseMapper::loadState(stream);
	fread(&reg[0], sizeof(reg[0]), 1, stream);		// misc
	fread(&reg[1], sizeof(reg[1]), 1, stream);		// misc
}