//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//////////////////////////////////////////////////////////////////////
// NesCartridge.h: interface for the NesCartridge class.
//////////////////////////////////////////////////////////////////////
// Class Description:  The NesCartridge class is responisble for loading
// NES files into memory.  Other components of the NES such as the CPU
// and the mappers look at the data contained in this class.
//////////////////////////////////////////////////////////////////////

#ifndef NES_CARTRIDGE
#define NES_CARTRIDGE

#include "Porting/basetypes.h"

// Constants that correspond to error codes
#define		INVALID_HEADER		-2	
#define		FILE_NOT_FOUND		-1
#define		OK					0

class NesCartridge  
{
public:
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesCartridge(unsigned char *&cartdata);
	NesCartridge();
	virtual ~NesCartridge();


	///////////////////////////////
	// Class Data Members	
	///////////////////////////////	
	struct iNESheader
	{
		char				name[3];	 	 // 'NES'
		Uint8				code;			 // $1A
		Uint8				PRGROM_count;    // PRG-ROM page count
		Uint8				CHRROM_count;    // CHR-ROM page count
		Uint8				ROM_Control_1;	 // ROM Control byte #1
		Uint8				ROM_Control_2;	 // ROM Control byte #2
		Uint8				blanks[8];		 // Blank spaces
		int					mapper;			 // Mapper number
	};
		
	// PRGROM_Pages is a pointer to an array of pointers that contain all
	// of the PRGROM contained in the cartridge...basically a 2 dimenional
	// array, like PRGROM_Pages[page num][16k]
	Uint8 ** PRGROM_Pages;

	// Array of 8K CHR-RAM pages from cartridge
    Uint8 **CHRROM_Pages;

	iNESheader header;

	// Status code that details load procedure
	int mintErrorCode;
};

#endif 
