//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// Nes PPU.h: interface for the NesPPU class.
//////////////////////////////////////////////////////////////////////
// Class Description:  The NesPPU class is reponsible for drawing the
// NES graphics to the "display" buffer that is a data member.  The
// PPU, with the help of a PPU Memory object, has its own area of
// memory and has many nuances when it comes to presenting graphics,
// most of which are thoroughly documented in the code.
//////////////////////////////////////////////////////////////////////

#ifndef NESPPU
#define NESPPU

#include "Porting/basetypes.h"
#include "NesPPUMemory.h"



class NesPPU  
{
public:

	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesPPU();
	virtual ~NesPPU();
	void Init();	
	virtual void calcScanLine();
	void clearDisplay();
	void drawBackGround();
	void drawSprites();	

	void saveState(FILE *stream);
	void loadState(FILE *stream);
	
	///////////////////////////////
	// Class Data Members	
	///////////////////////////////		
	NesPPUMemory *VRAM;			// PPU memory (VRAM) object

	// SPR-RAM, a special page of memory independent of the CPU and PPU memory
	unsigned char SPRRAM[0xFF + 1];

	// These "registers" values interact with the CPU in the RAM Read and Write methods
	// in the NesCPU class.
	struct PPU_regs
	{	
		Uint8 bytControlReg1;
		Uint8 bytControlReg2;
		Uint8 bytPPUStatus;		// Read only via $2002
		Uint8 bytSPRRAMAddr;		
		Uint8 bytVRAMIOReg;
		Uint16 tempVRAMAddr;	// "extra" VRAM address using for scrolling (aka the $2005 register)
		Uint16 VRAMAddr;		// Current address pointer for PPU reads/writes (aka the $2006 register)
	};
		
	PPU_regs regs;

	// These timers store exactly when an interrupt is triggered by the PPU.
	// The CPU uses these timers to actually raise the IRQ at the proper time
	// in CPU emulation
	struct {
		int  intSpr0HitPix;		// Sprite #0 hit (pixel)
		int  intSpr0HitSL;		// Sprite #0 hit (scanline)
		bool blnOtherIRQ;		// Other IRQ  (currently only used by MMC3)
		int  intOtherPix;		// Other IRQ pixel		
		int  intCurPPUCycle;	// Current PPU cycle
	} timers;

	unsigned char *display;				// PPU display (a 256x256 8 bit bitmap)	
	Uint16 mintCurScanLine;				// Scan line counter	
	Uint32 mPalette[64];				// 256 byte current NES palette
	bool mblnSpriteFlag;				// Sprite #0 hit flag, set by CPU at start of frame	
	int mintNumScanlines;				// Number of scanlines in a frame
	int mintNumVBlankScanlines;			// Number of VBlank scanlines in a frame
	
	// Structure used to store various drawing values
	struct
	{
		Uint8 bytXPixelOffset;		
	} draw;		
};

#endif
